@@ -234,8 +234,7 @@ _gst_va_msg_sendf (PTR resultPtr,
           break;
 
         case 'C':
-          args[++i] = COBJECT_NEW (va_arg (ap, PTR), _gst_nil_oop,
-				   _gst_c_object_class);
+          args[++i] = COBJECT_NEW (va_arg (ap, PTR));
 	  INC_ADD_OOP (args[i]);
           break;
 
@@ -260,7 +259,8 @@ _gst_va_msg_sendf (PTR resultPtr,
 	    ctype = _gst_type_name_to_oop (va_arg (ap, const char *));
 	    INC_ADD_OOP (ctype);
 
-	    args[++i] = COBJECT_NEW (va_arg (ap, PTR), ctype, _gst_nil_oop);
+	    args[++i] =
+	      _gst_c_object_new (va_arg (ap, PTR), ctype, _gst_nil_oop);
 
 	    INC_ADD_OOP (args[i]);
           }
@@ -271,14 +271,19 @@ _gst_va_msg_sendf (PTR resultPtr,
           {
     	    OOP ctype;
     	    ctype = va_arg (ap, OOP);
-    	    args[++i] = COBJECT_NEW (va_arg (ap, PTR), ctype, _gst_nil_oop);
+    	    args[++i] =
+    	      _gst_c_object_new (va_arg (ap, PTR), ctype, _gst_nil_oop);
 
 	    INC_ADD_OOP (args[i]);
           }
           break;
 
         case 'w':
-          args[++i] = char_new ((char) va_arg (ap, wchar_t));
+#if SIZEOF_WCHAR_T <= SIZEOF_INT
+          args[++i] = char_new ((wchar_t) va_arg (ap, int));
+#else
+          args[++i] = char_new ((wchar_t) va_arg (ap, wchar_t));
+#endif
 	  INC_ADD_OOP (args[i]);
           break;
 
@@ -306,7 +311,7 @@ _gst_va_msg_sendf (PTR resultPtr,
 
 	case 'C':
 	  *(PTR *) resultPtr =
-	    IS_NIL (result) ? NULL : cobject_value (result);
+	    IS_NIL (result) ? NULL : COBJECT_VALUE (result);
 	  break;
 
 	case 's':
@@ -587,7 +592,7 @@ _gst_c_object_to_oop (PTR co)
   if (co == NULL)
     return (_gst_nil_oop);
   else
-    return (INC_ADD_OOP (COBJECT_NEW (co, _gst_nil_oop, _gst_c_object_class)));
+    return (INC_ADD_OOP (COBJECT_NEW (co)));
 }
 
 void
@@ -596,7 +601,7 @@ _gst_set_c_object (OOP oop, PTR co)
   if (!_gst_smalltalk_initialized)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
-  set_cobject_value (oop, co);
+  SET_COBJECT_VALUE(oop, co);
 }
 
 
@@ -629,7 +634,7 @@ _gst_oop_to_c (OOP oop)
     return (0);
 
   else if (is_a_kind_of (OOP_CLASS (oop), _gst_c_object_class))
-    return ((long) cobject_value (oop));
+    return ((long) COBJECT_VALUE (oop));
 
   else
     return (0);
@@ -757,7 +762,7 @@ _gst_oop_to_c_object (OOP oop)
   if (IS_NIL (oop))
     return (NULL);
   else
-    return (cobject_value (oop));
+    return (COBJECT_VALUE (oop));
 }
 
 OOP
@@ -921,24 +926,6 @@ _gst_oop_at_put (OOP oop, size_t index, OOP new)
   index_oop_put (oop, index + 1, new);
   return old;
 }
-
-void *
-_gst_oop_indexed_base (OOP oop)
-{
-  if (!_gst_smalltalk_initialized)
-    _gst_initialize (NULL, NULL, GST_NO_TTY);
-
-  return OOP_TO_OBJ (oop)->data[OOP_FIXED_FIELDS (oop)];
-}
-
-enum gst_indexed_kind
-_gst_oop_indexed_kind (OOP oop)
-{
-  if (!_gst_smalltalk_initialized)
-    _gst_initialize (NULL, NULL, GST_NO_TTY);
-
-  return OOP_INSTANCE_SPEC (oop) & ISP_INDEXEDVARS;
-}
 
 
 
@@ -955,7 +942,7 @@ _gst_register_oop (OOP oop)
   oop_registry *node;
   oop_registry *entry = NULL;
 
-  if (!oop || IS_NIL (oop))
+  if (!oop)
     return (oop);
 
   while (*p)
@@ -991,7 +978,7 @@ _gst_unregister_oop (OOP oop)
 
   /* Speed things up, this will never be in the registry (but we allow
      it to simplify client code).  */
-  if (!oop || IS_NIL (oop))
+  if (!oop)
     return;
 
   while (entry)
