@@ -106,6 +106,10 @@
 #include <inttypes.h>
 #endif
 
+#ifdef HAVE_SIGSEGV_H
+#include "sigsegv.h"
+#endif
+
 #include "gst.h"
 
 /* Convenience macros to test the versions of GCC.  Note - they won't
@@ -580,7 +584,7 @@ extern OOP _gst_nil_oop
 #define ST_INT_SIZE        ((sizeof (PTR) * 8) - 2)
 #define MAX_ST_INT         ((1L << ST_INT_SIZE) - 1)
 #define MIN_ST_INT         ( ~MAX_ST_INT)
-#define INT_OVERFLOW(i)    ( (i) > MAX_ST_INT || (i) < MIN_ST_INT )
+#define INT_OVERFLOW(i)    (((i) ^ ((i) << 1)) < 0)
 #define OVERFLOWING_INT    (MAX_ST_INT + 1)
 
 #define INCR_INT(i)         ((OOP) (((intptr_t)i) + 2))	/* 1 << 1 */
