@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006,2007
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -81,7 +81,8 @@ enum
   PRIM_IS_PIPE,                 /* no args */
   PRIM_MK_TEMP,                 /* base: */
   PRIM_GET_CHARS_AT,            /* data:from:to:absOfs: */
-  PRIM_PUT_CHARS_AT             /* data:from:to:absOfs: */
+  PRIM_PUT_CHARS_AT,            /* data:from:to:absOfs: */
+  PRIM_SHUTDOWN_WRITE           /* shutdown */
 };
 
 /* These macros are used to quickly compute the number of words needed
@@ -180,6 +181,13 @@ typedef struct gst_block_context
 }
  *gst_block_context;
 
+typedef struct gst_continuation
+{
+  OBJ_HEADER;
+  OOP stack;
+}
+ *gst_continuation;
+
 /* The structure of various objects related to the process system.  */
 typedef struct gst_semaphore
 {
@@ -546,7 +554,7 @@ prim_table_entry;
 /* The checksum of the table of primitive numbers.  Right now it is an MD5,
    computed from part of the C source code of prims.inl.  We compare it when
    loading an image, to avoid having to reload the primitive table.  */
-extern int _gst_primitives_md5[4];
+extern unsigned char _gst_primitives_md5[16];
 
 /* The table of functions that implement the primitives.  */
 extern prim_table_entry _gst_primitive_table[NUM_PRIMITIVES];
@@ -569,4 +577,17 @@ extern void _gst_set_primitive_attributes (int primitive,
 extern void _gst_init_primitives () 
   ATTRIBUTE_HIDDEN;
 
+/* Get the value of internal variable whose number is INDEX; the
+   list of valid variables is in gstpub.h.  Return -1 if the index
+   is invalid.  */
+extern int _gst_get_var (enum gst_var_index index)
+  ATTRIBUTE_HIDDEN;
+
+/* Set the value of internal variable whose number is INDEX; the
+   list of valid variables is in gstpub.h.  Return -1 if the index
+   is invalid or the value is negative, otherwise return the previous
+   value.  */
+extern int _gst_set_var (enum gst_var_index index, int value)
+  ATTRIBUTE_HIDDEN;
+
 #endif /* GST_INTERP_H */
