@@ -635,17 +635,16 @@ _gst_signal_after (int deltaMilli,
     }
 }
 
+static SigHandler sigio_handler = SIG_IGN;
+
 void
 _gst_set_file_interrupt (int fd,
 			 SigHandler func)
 {
-  static SigHandler old_func;
-  if (func != old_func) 
+  if (func != sigio_handler) 
     {
-      old_func = func;
+      sigio_handler = func;
 
-      /* Trap SIGCHLD, it might reveal a POLLHUP event! */
-      _gst_set_signal_handler (SIGCHLD, func);
 #ifdef SIGPOLL
       _gst_set_signal_handler (SIGPOLL, func);
 #else
@@ -898,7 +897,7 @@ _gst_get_full_file_name (const char *name)
 		}
 
 	      buf = alloca (path_max);
-	      n = readlink (rpath, buf, path_max);
+	      n = readlink (rpath, buf, path_max - 1);
 	      if (n < 0)
 		{
 		  int saved_errno = errno;
@@ -1011,9 +1010,6 @@ _gst_file_is_executable (const char *fileName)
 static int executable_fd = -1;
 #endif
 
-/* The path to the executable, derived from argv[0].  */
-const char *_gst_executable_path = NULL;
-
 /* Tests whether a given pathname may belong to the executable.  */
 static mst_Boolean
 maybe_executable (const char *filename)
@@ -1047,8 +1043,8 @@ maybe_executable (const char *filename)
    Return NULL if unknown.  Guaranteed to work on Linux and Win32, Mac OS X.
    Likely to work on the other Unixes (maybe except BeOS), under most
    conditions.  */
-static char *
-find_executable (const char *argv0)
+char *
+_gst_find_executable (const char *argv0)
 {
 #if defined WIN32
   char location[MAX_PATH];
@@ -1100,13 +1096,16 @@ find_executable (const char *argv0)
      enclosed in brackets, which we cannot use here.  */
   {
     char buf[6 + 10 + 5];
-    char *location = alloca (path_max);
+    char *location = xmalloc (path_max + 1);
     ssize_t n;
 
     sprintf (buf, "/proc/%d/exe", getpid ());
     n = readlink (buf, location, path_max);
     if (n > 0 && location[0] != '[')
-      return location;
+      {
+        location[n] == '\0';
+        return location;
+      }
     if (executable_fd < 0)
       executable_fd = open (buf, O_RDONLY, 0);
   }
@@ -1161,46 +1160,6 @@ find_executable (const char *argv0)
 #endif
 }
 
-void
-_gst_set_executable_path (const char *argv0)
-{
-  _gst_executable_path = find_executable (argv0);
-}
-
-char *
-_gst_relocate_path (const char *path)
-{
-  const char *p;
-  char *s;
-
-  /* Detect absolute paths.  */
-#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
-  if ((path[0] && path[1] == ':')
-      || path[0] == '/' || path[0] == '\\')
-    return xstrdup (path);
-#else
-  if (path[0] == '/')
-    return xstrdup (path);
-#endif
-
-  /* Remove filename from executable path.  */
-  p = _gst_executable_path + strlen (_gst_executable_path);
-  do
-    --p;
-  while (p >= _gst_executable_path
-         && *p != '/'
-#if defined(MSDOS) || defined(WIN32) || defined(__OS2__)
-	 && *p != '\\'
-#endif
-	);
-  p++;
-
-  /* Now p points just past the last separator (if any).  */
-  s = alloca (p - _gst_executable_path + strlen (path) + 1);
-  sprintf (s, "%.*s%s", p - _gst_executable_path, _gst_executable_path, path);
-  return _gst_get_full_file_name (s);
-}
-
 
 /* Code to use PTY's did not work on Mac OS.  I'm keeping the Unix code,
    but it should not be used.  */
@@ -1720,6 +1679,10 @@ _gst_open_file (const char *filename,
 	oflags |= O_EXCL;
     }
 
+#ifdef O_CLOEXEC
+  oflags |= O_CLOEXEC;
+#endif
+
   if (create)
     fd = open (filename, oflags | access | O_CREAT, 0666);
   else
@@ -1728,7 +1691,7 @@ _gst_open_file (const char *filename,
   if (fd < 0)
     return -1;
 
-#ifdef FD_CLOEXEC
+#if defined FD_CLOEXEC && !defined O_CLOEXEC
   fcntl (fd, F_SETFD, fcntl (fd, F_GETFD, 0) | FD_CLOEXEC);
 #endif
 
@@ -1869,19 +1832,25 @@ _gst_full_write (int fd,
 }
 
 
-#ifdef HAVE_WAITPID
 static void
 sigchld_handler (int signum)
 {
+#ifdef HAVE_WAITPID
   int pid, status, serrno;
   serrno = errno;
   do
     pid = waitpid (-1, &status, WNOHANG);
   while (pid > 0);
   errno = serrno;
-}
 #endif
 
+  /* Pass it to the SIGIO handler, it might reveal a POLLHUP event.  */
+  if (sigio_handler != SIG_DFL && sigio_handler != SIG_IGN)
+    sigio_handler (signum);
+
+  _gst_set_signal_handler (SIGCHLD, sigchld_handler);
+}
+
 void
 _gst_init_sysdep (void)
 {
@@ -1892,10 +1861,7 @@ _gst_init_sysdep (void)
   tzset ();
 
   _gst_set_signal_handler (SIGPIPE, SIG_IGN);
-
-#ifdef HAVE_WAITPID
   _gst_set_signal_handler (SIGCHLD, sigchld_handler);
-#endif
 }
 
 void
